; DriveWait GUI v0.1 by Mark Jones 2006 (gzscuqn02 AT sneakemail D0T com)
; Waits 16 seconds for drive to respond, then displays error message.
; Executes supplied arguments if drive comes on-line. Returns errorlevel 
; 0 for success and 1 for failure.

include masm32rt.inc                    ; standard compile-time libs

.const
    szMutex         db  "Global\DWv0.1",0,0,0   ; instance marker
    szDWG           db  "DriveWaitG v0.1",0     ; dialog text
    szOpen          db  "open",0,0,0,0  ; file_open
.data?
    hFolder         dd  ?               ; handle to drive
    hMutex          dd  ?               ; handle to instance
    szProg          db  128 dup(?)      ; file to run
    szArgs          db  256 dup(?)      ; arguments
    nCount          db  ?               ; byte used to retry drive 16 times
    nErrLvl         db  ?               ; byte used to convey errorlevel
.data
    szDest          db  "X:\",0         ; removable drive
    szPowerUpDrive  db  "Please connect drive ", 7 dup (0)

.code
start:
    invoke CreateMutex,0,1,addr szMutex ; only allow one instance
    mov hMutex,eax
    .if eax != NULL && FUNC( GetLastError ) == ERROR_ALREADY_EXISTS
err:    invoke Beep,100,1000            ; error sound
        invoke ExitProcess,1            ; exit with error!
    .endif

getargs:                                ; process arguments
    push esi                            ; save esi
    push edi                            ; and edi
    cld                                 ; direction flag must be cleared
    call GetCommandLine                 ; returns pointer to string
    mov esi,eax                         ; esi = "\DWG.exe" X arg, arg...
    lea edi,szDest                      ; edi = "X:\"
    cmp byte ptr [esi],22h              ; is the first byte a quote?
    jz @F                               ; if so, process fully
    cmp byte ptr [esi+3],00             ; are there no arguments in dos-mode?
    jz err                              ; if none, exit with a beep
    add esi,4                           ; else skip over "DWG "
    jmp copydrv                         ; and copy drive letter
@@: inc esi                             ; esi = next source byte
    cmp byte ptr [esi],22h              ; test for trailing " character
    jnz @B                              ; loop back until found
    add esi,2                           ; skip space
    cmp byte ptr [esi],00               ; are there no arguments in win-mode?
    jz err                              ; if none, exit with a beep
copydrv:
    movsb                               ; copy byte edi <-- esi, both+1
    inc esi                             ; skip space
    lea edi,szProg                      ; edi = file to run
    cmp byte ptr [esi],22h              ; is filename quoted?
    jnz spaced                          ; if not, look for next space
    movsb                               ; copy initial quote char
@@: movsb                               ; copy a byte
    cmp byte ptr [esi-1],22h            ; wait for ending " to be copied
    jnz @B                              ; loop until copied
    inc esi                             ; skip space
    jmp args                            ; then copy args
spaced:
    movsb                               ; copy a byte
    cmp byte ptr [esi-1],00             ; look for null
    jz ready                            ; break out if null found
    cmp byte ptr [esi-1]," "            ; look for the " " after filename
    jnz spaced                          ; loop until found
args:
    lea edi,szArgs                      ; edi = arguments
@@: movsb                               ; copy argument byte
    cmp byte ptr [esi-1],00             ; test for a copied null
    jnz @B                              ; loop until done
    pop edi                             ; restore edi
    pop esi                             ; and esi
ready:
    mov eax,dword ptr [szDest]          ; fetch 4 text bytes into eax
    mov dword ptr [szPowerUpDrive+21],eax   ; copy into display string

    invoke CreateFile,addr szDest,\     ; try opening destination drive
                      FILE_LIST_DIRECTORY,\
                      FILE_SHARE_READ+FILE_SHARE_WRITE+FILE_SHARE_DELETE,\
                      NULL,\
                      OPEN_EXISTING,\
                      FILE_FLAG_BACKUP_SEMANTICS,\
                      NULL
    mov hFolder,eax                     ; save "folder" handle
    .if FUNC(GetLastError) == ERROR_PATH_NOT_FOUND
        invoke CloseHandle,hFolder      ; close file handle
        invoke Sleep,1000               ; wait 1 second
        sub nCount,16                   ; decrease counter by 16
        jnz ready                       ; repeat 16 times
        invoke MessageBox,0,addr szPowerUpDrive,addr szDWG,\
                          MB_RETRYCANCEL+MB_SYSTEMMODAL
        .if eax==IDCANCEL
            mov nErrLvl,1               ; mark as error
            jmp endit                   ; exit
        .else
            jmp ready                   ; try again
        .endif
    .else                               ; drive found, run command + args
        ;invoke Sleep,2000               ; give 2 seconds for system to stabilize
        invoke ShellExecute,0,addr szOpen,addr szProg,addr szArgs,0,SW_SHOW
        cmp eax,32                      ; if no error,
        ja endit                        ; exit
        mov nErrLvl,1                   ; else mark as error
    .endif                              ; and fall through
endit:
    invoke CloseHandle,hFolder          ; close file handle
    invoke CloseHandle,hMutex           ; close mutex handle
    invoke ExitProcess,nErrLvl          ; exit gracefully with errorlevel
end start
